package io.github.null2264.tsukumogami.core.commands

class StringParser(private val string: String) {
    private var currentIndex = 0
    private var lastPeekIndex = 0

    fun hasNext() = currentIndex < string.length

    private fun parse(peek: Boolean, naive: Boolean = false): String {
        check(hasNext())

        val currentToken = StringBuilder()
        var inQuotes = false
        val currentIndexInitial = currentIndex

        while (hasNext()) {
            val char = string[currentIndex]
            currentIndex++

            when (char) {
                '"' -> {
                    if (naive) {
                        currentToken.append(char)
                        continue
                    }

                    if (inQuotes) {
                        // Closing quote: add the accumulated token
                        break
                    } else {
                        if (currentToken.isNotEmpty()) {
                            // Did not start with quote, probably something like: hello"world"
                            currentToken.append(char)
                            continue
                        }
                        // Opening quote:
                        inQuotes = true
                    }
                }

                ' ' -> {
                    if (inQuotes) {
                        // Space inside quotes: append it
                        currentToken.append(char)
                    } else {
                        // Space outside quotes: separator
                        if (currentToken.isNotEmpty()) {
                            break
                        }
                    }
                }

                else -> {
                    // Any other character: append it
                    currentToken.append(char)
                }
            }
        }
        //check(!inQuotes)
        if (peek) {
            lastPeekIndex = currentIndex
            currentIndex = currentIndexInitial
        }
        return currentToken.toString()
    }

    fun parseNext(naive: Boolean = false): String = parse(false, naive)
    fun peekNext(naive: Boolean = false): String = parse(true, naive)
    fun next() {
        currentIndex = lastPeekIndex
    }
}
