package io.github.null2264.tsukumogami.core.commands.converters.impl

import dev.kord.common.entity.Snowflake
import dev.kord.core.entity.User
import io.github.null2264.tsukumogami.core.Context
import io.github.null2264.tsukumogami.core.commands.StringParser
import io.github.null2264.tsukumogami.core.commands.converters.Converter
import io.github.null2264.tsukumogami.core.exceptions.CommandException
import io.github.null2264.tsukumogami.core.ext.users
import kotlinx.coroutines.flow.firstOrNull

class UserConverter : Converter<User>() {

    override var isGreedy: Boolean
        get() = false
        set(_) { error("UserConverter can't be greedy") }

    override lateinit var parsed: User

    override suspend fun consume(context: Context, parser: StringParser): Boolean {
        if (!parser.hasNext()) throw CommandException("User is not specified")
        val input = parser.peekNext()

        if (input.equals("me", true)) {
            val user = context.author
            if (user != null) {
                this.parsed = user
                parser.next()
                return true
            }
        }
        if (input.equals("you", true)) {
            this.parsed = context.bot.client.getSelf()
            parser.next()
            return true
        }

        this.parsed = context.findUser(input) ?:
            throw CommandException("User not found")

        parser.next()
        return true
    }

    private suspend fun Context.findUser(arg: String): User? =
        if (arg.startsWith("<@") && arg.endsWith(">")) {
            val id: String = arg.substring(2, arg.length - 1).replace("!", "")

            try {
                bot.client.getUser(Snowflake(id))
            } catch (_: NumberFormatException) {
                throw CommandException("Invalid user ID")
            }
        } else {
            try {
                bot.client.getUser(Snowflake(arg))
            } catch (_: NumberFormatException) {
                if (!arg.contains("#")) {
                    null
                } else {
                    bot.client.users.firstOrNull { user ->
                        user.tag.equals(arg, true)
                    }
                }
            }
        }
}
